# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types.fleetify import (
    DocumentTemplateListResponse,
    DocumentTemplateCreateResponse,
    DocumentTemplateDeleteResponse,
    DocumentTemplateUpdateResponse,
    DocumentTemplateRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDocumentTemplates:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: NextbillionSDK) -> None:
        document_template = client.fleetify.document_templates.create(
            key="key",
            content=[
                {
                    "label": '"label": "Specify Completion Time"',
                    "type": "`string`",
                }
            ],
            name="name",
        )
        assert_matches_type(DocumentTemplateCreateResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: NextbillionSDK) -> None:
        response = client.fleetify.document_templates.with_raw_response.create(
            key="key",
            content=[
                {
                    "label": '"label": "Specify Completion Time"',
                    "type": "`string`",
                }
            ],
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document_template = response.parse()
        assert_matches_type(DocumentTemplateCreateResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: NextbillionSDK) -> None:
        with client.fleetify.document_templates.with_streaming_response.create(
            key="key",
            content=[
                {
                    "label": '"label": "Specify Completion Time"',
                    "type": "`string`",
                }
            ],
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document_template = response.parse()
            assert_matches_type(DocumentTemplateCreateResponse, document_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: NextbillionSDK) -> None:
        document_template = client.fleetify.document_templates.retrieve(
            id="id",
            key="key",
        )
        assert_matches_type(DocumentTemplateRetrieveResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: NextbillionSDK) -> None:
        response = client.fleetify.document_templates.with_raw_response.retrieve(
            id="id",
            key="key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document_template = response.parse()
        assert_matches_type(DocumentTemplateRetrieveResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: NextbillionSDK) -> None:
        with client.fleetify.document_templates.with_streaming_response.retrieve(
            id="id",
            key="key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document_template = response.parse()
            assert_matches_type(DocumentTemplateRetrieveResponse, document_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.fleetify.document_templates.with_raw_response.retrieve(
                id="",
                key="key",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: NextbillionSDK) -> None:
        document_template = client.fleetify.document_templates.update(
            id="id",
            key="key",
        )
        assert_matches_type(DocumentTemplateUpdateResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: NextbillionSDK) -> None:
        document_template = client.fleetify.document_templates.update(
            id="id",
            key="key",
            content=[
                {
                    "label": '"label": "Specify Completion Time"',
                    "type": "`string`",
                    "meta": {
                        "options": [
                            {
                                "label": '"label": "Option 1"',
                                "value": '"value": "Car"',
                            }
                        ]
                    },
                    "name": '"name" : "Completion DateTime"',
                    "required": True,
                    "validation": {
                        "max": 0,
                        "max_items": 0,
                        "min": 0,
                        "min_items": 0,
                    },
                }
            ],
            name="name",
        )
        assert_matches_type(DocumentTemplateUpdateResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: NextbillionSDK) -> None:
        response = client.fleetify.document_templates.with_raw_response.update(
            id="id",
            key="key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document_template = response.parse()
        assert_matches_type(DocumentTemplateUpdateResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: NextbillionSDK) -> None:
        with client.fleetify.document_templates.with_streaming_response.update(
            id="id",
            key="key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document_template = response.parse()
            assert_matches_type(DocumentTemplateUpdateResponse, document_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.fleetify.document_templates.with_raw_response.update(
                id="",
                key="key",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: NextbillionSDK) -> None:
        document_template = client.fleetify.document_templates.list(
            key="key",
        )
        assert_matches_type(DocumentTemplateListResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: NextbillionSDK) -> None:
        response = client.fleetify.document_templates.with_raw_response.list(
            key="key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document_template = response.parse()
        assert_matches_type(DocumentTemplateListResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: NextbillionSDK) -> None:
        with client.fleetify.document_templates.with_streaming_response.list(
            key="key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document_template = response.parse()
            assert_matches_type(DocumentTemplateListResponse, document_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_delete(self, client: NextbillionSDK) -> None:
        document_template = client.fleetify.document_templates.delete(
            id="id",
            key="key",
        )
        assert_matches_type(DocumentTemplateDeleteResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete(self, client: NextbillionSDK) -> None:
        response = client.fleetify.document_templates.with_raw_response.delete(
            id="id",
            key="key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document_template = response.parse()
        assert_matches_type(DocumentTemplateDeleteResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete(self, client: NextbillionSDK) -> None:
        with client.fleetify.document_templates.with_streaming_response.delete(
            id="id",
            key="key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document_template = response.parse()
            assert_matches_type(DocumentTemplateDeleteResponse, document_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_delete(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.fleetify.document_templates.with_raw_response.delete(
                id="",
                key="key",
            )


class TestAsyncDocumentTemplates:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncNextbillionSDK) -> None:
        document_template = await async_client.fleetify.document_templates.create(
            key="key",
            content=[
                {
                    "label": '"label": "Specify Completion Time"',
                    "type": "`string`",
                }
            ],
            name="name",
        )
        assert_matches_type(DocumentTemplateCreateResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.fleetify.document_templates.with_raw_response.create(
            key="key",
            content=[
                {
                    "label": '"label": "Specify Completion Time"',
                    "type": "`string`",
                }
            ],
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document_template = await response.parse()
        assert_matches_type(DocumentTemplateCreateResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.fleetify.document_templates.with_streaming_response.create(
            key="key",
            content=[
                {
                    "label": '"label": "Specify Completion Time"',
                    "type": "`string`",
                }
            ],
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document_template = await response.parse()
            assert_matches_type(DocumentTemplateCreateResponse, document_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        document_template = await async_client.fleetify.document_templates.retrieve(
            id="id",
            key="key",
        )
        assert_matches_type(DocumentTemplateRetrieveResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.fleetify.document_templates.with_raw_response.retrieve(
            id="id",
            key="key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document_template = await response.parse()
        assert_matches_type(DocumentTemplateRetrieveResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.fleetify.document_templates.with_streaming_response.retrieve(
            id="id",
            key="key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document_template = await response.parse()
            assert_matches_type(DocumentTemplateRetrieveResponse, document_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.fleetify.document_templates.with_raw_response.retrieve(
                id="",
                key="key",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncNextbillionSDK) -> None:
        document_template = await async_client.fleetify.document_templates.update(
            id="id",
            key="key",
        )
        assert_matches_type(DocumentTemplateUpdateResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        document_template = await async_client.fleetify.document_templates.update(
            id="id",
            key="key",
            content=[
                {
                    "label": '"label": "Specify Completion Time"',
                    "type": "`string`",
                    "meta": {
                        "options": [
                            {
                                "label": '"label": "Option 1"',
                                "value": '"value": "Car"',
                            }
                        ]
                    },
                    "name": '"name" : "Completion DateTime"',
                    "required": True,
                    "validation": {
                        "max": 0,
                        "max_items": 0,
                        "min": 0,
                        "min_items": 0,
                    },
                }
            ],
            name="name",
        )
        assert_matches_type(DocumentTemplateUpdateResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.fleetify.document_templates.with_raw_response.update(
            id="id",
            key="key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document_template = await response.parse()
        assert_matches_type(DocumentTemplateUpdateResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.fleetify.document_templates.with_streaming_response.update(
            id="id",
            key="key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document_template = await response.parse()
            assert_matches_type(DocumentTemplateUpdateResponse, document_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.fleetify.document_templates.with_raw_response.update(
                id="",
                key="key",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncNextbillionSDK) -> None:
        document_template = await async_client.fleetify.document_templates.list(
            key="key",
        )
        assert_matches_type(DocumentTemplateListResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.fleetify.document_templates.with_raw_response.list(
            key="key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document_template = await response.parse()
        assert_matches_type(DocumentTemplateListResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.fleetify.document_templates.with_streaming_response.list(
            key="key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document_template = await response.parse()
            assert_matches_type(DocumentTemplateListResponse, document_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete(self, async_client: AsyncNextbillionSDK) -> None:
        document_template = await async_client.fleetify.document_templates.delete(
            id="id",
            key="key",
        )
        assert_matches_type(DocumentTemplateDeleteResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.fleetify.document_templates.with_raw_response.delete(
            id="id",
            key="key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document_template = await response.parse()
        assert_matches_type(DocumentTemplateDeleteResponse, document_template, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.fleetify.document_templates.with_streaming_response.delete(
            id="id",
            key="key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document_template = await response.parse()
            assert_matches_type(DocumentTemplateDeleteResponse, document_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.fleetify.document_templates.with_raw_response.delete(
                id="",
                key="key",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types.skynet import SearchResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSearch:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_around(self, client: NextbillionSDK) -> None:
        search = client.skynet.search.around(
            center="56.597801,43.967836",
            key="key=API_KEY",
            radius=0,
        )
        assert_matches_type(SearchResponse, search, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_around_with_all_params(self, client: NextbillionSDK) -> None:
        search = client.skynet.search.around(
            center="56.597801,43.967836",
            key="key=API_KEY",
            radius=0,
            filter="filter=tag:delivery,truck",
            include_all_of_attributes="include_all_of_attributes=vehicle_type:pickup_truck|driver_name:John",
            include_any_of_attributes="include_any_of_attributes=vehicle_type:pickup_truck|driver_name:John",
            max_search_limit=True,
            pn=0,
            ps=100,
            sort_by="`distance`",
            sort_destination="sort_destination= 34.0241,-118.2550",
            sort_driving_mode="`car`",
        )
        assert_matches_type(SearchResponse, search, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_around(self, client: NextbillionSDK) -> None:
        response = client.skynet.search.with_raw_response.around(
            center="56.597801,43.967836",
            key="key=API_KEY",
            radius=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        search = response.parse()
        assert_matches_type(SearchResponse, search, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_around(self, client: NextbillionSDK) -> None:
        with client.skynet.search.with_streaming_response.around(
            center="56.597801,43.967836",
            key="key=API_KEY",
            radius=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            search = response.parse()
            assert_matches_type(SearchResponse, search, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_bound(self, client: NextbillionSDK) -> None:
        search = client.skynet.search.bound(
            bound="bounds=44.7664,-0.6941|44.9206,-0.4639",
            key="key=API_KEY",
        )
        assert_matches_type(SearchResponse, search, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_bound_with_all_params(self, client: NextbillionSDK) -> None:
        search = client.skynet.search.bound(
            bound="bounds=44.7664,-0.6941|44.9206,-0.4639",
            key="key=API_KEY",
            filter="filter=tag:delivery,truck",
            include_all_of_attributes="include_all_of_attributes=vehicle_type:pickup_truck|driver_name:John",
            include_any_of_attributes="include_any_of_attributes=vehicle_type:pickup_truck|driver_name:John",
            max_search_limit=True,
            pn=0,
            ps=100,
            sort_by="`distance`",
            sort_destination="sort_destination= 34.0241,-118.2550",
            sort_driving_mode="`car`",
        )
        assert_matches_type(SearchResponse, search, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_bound(self, client: NextbillionSDK) -> None:
        response = client.skynet.search.with_raw_response.bound(
            bound="bounds=44.7664,-0.6941|44.9206,-0.4639",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        search = response.parse()
        assert_matches_type(SearchResponse, search, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_bound(self, client: NextbillionSDK) -> None:
        with client.skynet.search.with_streaming_response.bound(
            bound="bounds=44.7664,-0.6941|44.9206,-0.4639",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            search = response.parse()
            assert_matches_type(SearchResponse, search, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncSearch:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_around(self, async_client: AsyncNextbillionSDK) -> None:
        search = await async_client.skynet.search.around(
            center="56.597801,43.967836",
            key="key=API_KEY",
            radius=0,
        )
        assert_matches_type(SearchResponse, search, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_around_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        search = await async_client.skynet.search.around(
            center="56.597801,43.967836",
            key="key=API_KEY",
            radius=0,
            filter="filter=tag:delivery,truck",
            include_all_of_attributes="include_all_of_attributes=vehicle_type:pickup_truck|driver_name:John",
            include_any_of_attributes="include_any_of_attributes=vehicle_type:pickup_truck|driver_name:John",
            max_search_limit=True,
            pn=0,
            ps=100,
            sort_by="`distance`",
            sort_destination="sort_destination= 34.0241,-118.2550",
            sort_driving_mode="`car`",
        )
        assert_matches_type(SearchResponse, search, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_around(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.search.with_raw_response.around(
            center="56.597801,43.967836",
            key="key=API_KEY",
            radius=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        search = await response.parse()
        assert_matches_type(SearchResponse, search, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_around(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.search.with_streaming_response.around(
            center="56.597801,43.967836",
            key="key=API_KEY",
            radius=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            search = await response.parse()
            assert_matches_type(SearchResponse, search, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_bound(self, async_client: AsyncNextbillionSDK) -> None:
        search = await async_client.skynet.search.bound(
            bound="bounds=44.7664,-0.6941|44.9206,-0.4639",
            key="key=API_KEY",
        )
        assert_matches_type(SearchResponse, search, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_bound_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        search = await async_client.skynet.search.bound(
            bound="bounds=44.7664,-0.6941|44.9206,-0.4639",
            key="key=API_KEY",
            filter="filter=tag:delivery,truck",
            include_all_of_attributes="include_all_of_attributes=vehicle_type:pickup_truck|driver_name:John",
            include_any_of_attributes="include_any_of_attributes=vehicle_type:pickup_truck|driver_name:John",
            max_search_limit=True,
            pn=0,
            ps=100,
            sort_by="`distance`",
            sort_destination="sort_destination= 34.0241,-118.2550",
            sort_driving_mode="`car`",
        )
        assert_matches_type(SearchResponse, search, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_bound(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.search.with_raw_response.bound(
            bound="bounds=44.7664,-0.6941|44.9206,-0.4639",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        search = await response.parse()
        assert_matches_type(SearchResponse, search, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_bound(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.search.with_streaming_response.bound(
            bound="bounds=44.7664,-0.6941|44.9206,-0.4639",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            search = await response.parse()
            assert_matches_type(SearchResponse, search, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .routes import (
    RoutesResource,
    AsyncRoutesResource,
    RoutesResourceWithRawResponse,
    AsyncRoutesResourceWithRawResponse,
    RoutesResourceWithStreamingResponse,
    AsyncRoutesResourceWithStreamingResponse,
)
from .fleetify import (
    FleetifyResource,
    AsyncFleetifyResource,
    FleetifyResourceWithRawResponse,
    AsyncFleetifyResourceWithRawResponse,
    FleetifyResourceWithStreamingResponse,
    AsyncFleetifyResourceWithStreamingResponse,
)
from .document_templates import (
    DocumentTemplatesResource,
    AsyncDocumentTemplatesResource,
    DocumentTemplatesResourceWithRawResponse,
    AsyncDocumentTemplatesResourceWithRawResponse,
    DocumentTemplatesResourceWithStreamingResponse,
    AsyncDocumentTemplatesResourceWithStreamingResponse,
)

__all__ = [
    "RoutesResource",
    "AsyncRoutesResource",
    "RoutesResourceWithRawResponse",
    "AsyncRoutesResourceWithRawResponse",
    "RoutesResourceWithStreamingResponse",
    "AsyncRoutesResourceWithStreamingResponse",
    "DocumentTemplatesResource",
    "AsyncDocumentTemplatesResource",
    "DocumentTemplatesResourceWithRawResponse",
    "AsyncDocumentTemplatesResourceWithRawResponse",
    "DocumentTemplatesResourceWithStreamingResponse",
    "AsyncDocumentTemplatesResourceWithStreamingResponse",
    "FleetifyResource",
    "AsyncFleetifyResource",
    "FleetifyResourceWithRawResponse",
    "AsyncFleetifyResourceWithRawResponse",
    "FleetifyResourceWithStreamingResponse",
    "AsyncFleetifyResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .pagination import Pagination
from .asset.track_location import TrackLocation

__all__ = ["SearchResponse", "Data", "DataAsset", "DataAssetRankingInfo"]


class DataAssetRankingInfo(BaseModel):
    distance: Optional[float] = None
    """Driving distance between the asset and the `sort_destination`."""

    duration: Optional[float] = None
    """Driving duration between the asset and the `sort_destination`.

    Please note this field in not returned in the response when
    `sort_by = straight_distance` .
    """

    index: Optional[int] = None
    """Index of the ranked asset. The index value starts from 0."""


class DataAsset(BaseModel):
    id: Optional[str] = None
    """
    ID of `asset` which was last located inside the specified area in the input
    request. This is the same ID that was generated/provided at the time of creating
    the `asset`.
    """

    created_at: Optional[int] = None
    """
    A UNIX timestamp in seconds representing the time at which the `asset` was
    created.
    """

    description: Optional[str] = None
    """Description of the `asset`.

    The value would be the same as that provided for the `description` parameter at
    the time of creating or updating the `asset`.
    """

    latest_location: Optional[TrackLocation] = None
    """An object with details of the tracked location.

    Please note that if there are no tracking records for an asset, no location data
    will be returned.
    """

    meta_data: Optional[object] = None
    """Any valid json object data.

    Can be used to save customized data. Max size is 65kb.
    """

    name: Optional[str] = None
    """Name of `asset`.

    The value would be the same as that provided for the `name` parameter at the
    time of creating or updating the `asset`.
    """

    ranking_info: Optional[DataAssetRankingInfo] = None
    """
    An object returning the sorting details of the asset as per the configuration
    specified in the input.
    """

    tags: Optional[List[str]] = None
    """
    **This parameter will be deprecated soon! Please move existing `tags` to
    `attributes` parameter.**

    Tags associated with the `asset`.
    """

    tracked_at: Optional[int] = None
    """
    A UNIX epoch timestamp in seconds representing the last time when the `asset`
    was tracked.
    """

    updated_at: Optional[int] = None
    """
    A UNIX timestamp in seconds representing the time at which the `asset` was last
    updated.
    """


class Data(BaseModel):
    assets: Optional[List[DataAsset]] = None
    """An array of objects with details of the asset(s) returned in the search result.

    Each object represents one `asset`
    """

    page: Optional[Pagination] = None
    """An object with pagination details of the search results.

    Use this object to implement pagination in your application.
    """


class SearchResponse(BaseModel):
    data: Optional[Data] = None
    """A data object containing the search result."""

    message: Optional[str] = None
    """Displays the error message in case of a failed request.

    If the request is successful, this field is not present in the response.
    """

    status: Optional[str] = None
    """A string indicating the state of the response.

    On successful responses, the value will be `Ok`. Indicative error messages are
    returned for different errors. See the [API Error Codes](#api-error-codes)
    section below for more information.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from .batch import (
    BatchResource,
    AsyncBatchResource,
    BatchResourceWithRawResponse,
    AsyncBatchResourceWithRawResponse,
    BatchResourceWithStreamingResponse,
    AsyncBatchResourceWithStreamingResponse,
)
from ...types import (
    geofence_list_params,
    geofence_create_params,
    geofence_delete_params,
    geofence_update_params,
    geofence_contains_params,
    geofence_retrieve_params,
)
from .console import (
    ConsoleResource,
    AsyncConsoleResource,
    ConsoleResourceWithRawResponse,
    AsyncConsoleResourceWithRawResponse,
    ConsoleResourceWithStreamingResponse,
    AsyncConsoleResourceWithStreamingResponse,
)
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.skynet.simple_resp import SimpleResp
from ...types.geofence_list_response import GeofenceListResponse
from ...types.geofence_create_response import GeofenceCreateResponse
from ...types.geofence_contains_response import GeofenceContainsResponse
from ...types.geofence_retrieve_response import GeofenceRetrieveResponse

__all__ = ["GeofenceResource", "AsyncGeofenceResource"]


class GeofenceResource(SyncAPIResource):
    @cached_property
    def console(self) -> ConsoleResource:
        return ConsoleResource(self._client)

    @cached_property
    def batch(self) -> BatchResource:
        return BatchResource(self._client)

    @cached_property
    def with_raw_response(self) -> GeofenceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return GeofenceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GeofenceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return GeofenceResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        key: str,
        type: Literal["`circle`", "`polygon`", "`isochrone`"],
        circle: geofence_create_params.Circle | NotGiven = NOT_GIVEN,
        custom_id: str | NotGiven = NOT_GIVEN,
        isochrone: geofence_create_params.Isochrone | NotGiven = NOT_GIVEN,
        meta_data: object | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        polygon: geofence_create_params.Polygon | NotGiven = NOT_GIVEN,
        tags: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GeofenceCreateResponse:
        """
        Create a geofence

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          type: Specify the type of the geofence that is being created.

          circle: Provide the details to create a circular geofence. Please note that this object
              is mandatory when `type` is `circle`. When the `type` is not `circle`, the
              properties of this object will be ignored while creating the geofence.

          custom_id: Set an unique ID for the new geofence. If not provided, an ID will be
              automatically generated in UUID format. A valid `custom_id` can contain letters,
              numbers, "-", & "\\__" only.

              Please note that the ID of a geofence can not be changed once it is created.

          isochrone: Provide the details to create an isochrone based geofence. Use this object when
              `type` is `isochrone`. When the `type` is not `isochrone`, the properties of
              this object will be ignored while creating the geofence.

          meta_data: Metadata of the geofence. Use this field to define custom attributes that
              provide more context and information about the geofence being created like
              country, group ID etc.

              The data being added should be in valid JSON object format (i.e. `key` and
              `value` pairs). Max size allowed for the object is 65kb.

          name: Name of the geofence. Use this field to assign a meaningful, custom name to the
              geofence being created.

          polygon: Provide the details to create a custom polygon type of geofence. Please note
              that this object is mandatory when `type` is `polygon`. When the `type` is not
              `polygon`, the properties of this object will be ignored while creating the
              geofence.

              Self-intersecting polygons or polygons containing other polygons are invalid and
              will be removed while processing the request.

              Area of the polygon should be less than 2000 km<sup>2</sup>.

          tags: An array of strings to associate multiple tags to the geofence. `tags` can be
              used to search or filter geofences (using `Get Geofence List` method).

              Create valid `tags` using a string consisting of alphanumeric characters (A-Z,
              a-z, 0-9) along with the underscore ('\\__') and hyphen ('-') symbols.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/geofence",
            body=maybe_transform(
                {
                    "type": type,
                    "circle": circle,
                    "custom_id": custom_id,
                    "isochrone": isochrone,
                    "meta_data": meta_data,
                    "name": name,
                    "polygon": polygon,
                    "tags": tags,
                },
                geofence_create_params.GeofenceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, geofence_create_params.GeofenceCreateParams),
            ),
            cast_to=GeofenceCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GeofenceRetrieveResponse:
        """
        Get a Geofence

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/geofence/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, geofence_retrieve_params.GeofenceRetrieveParams),
            ),
            cast_to=GeofenceRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        key: str,
        circle: geofence_update_params.Circle | NotGiven = NOT_GIVEN,
        isochrone: geofence_update_params.Isochrone | NotGiven = NOT_GIVEN,
        meta_data: object | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        polygon: geofence_update_params.Polygon | NotGiven = NOT_GIVEN,
        tags: List[str] | NotGiven = NOT_GIVEN,
        type: Literal["`circle`", "`polygon`", "`isochrone`"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Update a Geofence

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          circle: Use this object to update details of a circular geofence. Please note that this
              object is mandatory only when `type` is `circle`. When the `type` is not
              `circle`, the properties of this object will be ignored while creating the
              geofence.

          isochrone: Use this object to update details of an isochrone based geofence. Please note
              that this object is mandatory only when `type` is `isochrone`. When the `type`
              is not `isochrone`, the properties of this object will be ignored while creating
              the geofence.

          meta_data: Updated the `meta_data` associated with a geofence. Use this field to define
              custom attributes that provide more context and information about the geofence
              being updated like country, group ID etc.

              The data being added should be in valid JSON object format (i.e. `key` and
              `value` pairs). Max size allowed for the object is 65kb.

          name: Use this parameter to update the `name` of a geofence. Users can assign
              meaningful custom names to their geofences.

          polygon: Use this object to update details of a custom polygon geofence. Please note that
              this object is mandatory only when `type` is `polygon`. When the `type` is not
              `polygon`, the properties of this object will be ignored while creating the
              geofence.

              Self-intersecting polygons or polygons containing other polygons are invalid and
              will be removed while processing the request.

              Area of the polygon should be less than 2000 km<sup>2</sup>.

          tags: Use this parameter to add/modify one or multiple `tags` of a geofence. `tags`
              can be used to search or filter geofences (using `Get Geofence List` method).

              Valid values for updating `tags` consist of alphanumeric characters (A-Z, a-z,
              0-9) along with the underscore ('\\__') and hyphen ('-') symbols.

          type: Use this parameter to update the `type` of a geofence. Please note that you will
              need to provide required details for creating a geofence of the new `type`.
              Check other parameters of this method to know more.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/geofence/{id}",
            body=maybe_transform(
                {
                    "circle": circle,
                    "isochrone": isochrone,
                    "meta_data": meta_data,
                    "name": name,
                    "polygon": polygon,
                    "tags": tags,
                    "type": type,
                },
                geofence_update_params.GeofenceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, geofence_update_params.GeofenceUpdateParams),
            ),
            cast_to=SimpleResp,
        )

    def list(
        self,
        *,
        key: str,
        pn: int | NotGiven = NOT_GIVEN,
        ps: int | NotGiven = NOT_GIVEN,
        tags: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GeofenceListResponse:
        """
        Get Geofence List

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          pn: Denotes page number. Use this along with the `ps` parameter to implement
              pagination for your searched results. This parameter does not have a maximum
              limit but would return an empty response in case a higher value is provided when
              the result-set itself is smaller.

          ps: Denotes number of search results per page. Use this along with the `pn`
              parameter to implement pagination for your searched results.

          tags: Comma (`,`) separated list of `tags` which will be used to filter the geofences.

              Please note only the geofences which have all the `tags` added to this parameter
              will be included in the result. This parameter can accept a string with a
              maximum length of 256 characters.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/geofence/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "pn": pn,
                        "ps": ps,
                        "tags": tags,
                    },
                    geofence_list_params.GeofenceListParams,
                ),
            ),
            cast_to=GeofenceListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Delete a Geofence

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/geofence/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, geofence_delete_params.GeofenceDeleteParams),
            ),
            cast_to=SimpleResp,
        )

    def contains(
        self,
        *,
        key: str,
        locations: str,
        geofences: str | NotGiven = NOT_GIVEN,
        verbose: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GeofenceContainsResponse:
        """
        Geofence Contains

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          locations: Pipe (|) separated coordinates, in [latitude,longitude] format, of the locations
              to be searched against the geofences.

          geofences: A `,` separated list geofence IDs against which the `locations` will be
              searched. If not provided, then the 'locations' will be searched against all
              your existing geofences.

              Maximum length of the string can be 256 characters.

          verbose: When `true`, an array with detailed information of geofences is returned. When
              `false`, an array containing only the IDs of the geofences is returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/geofence/contain",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "locations": locations,
                        "geofences": geofences,
                        "verbose": verbose,
                    },
                    geofence_contains_params.GeofenceContainsParams,
                ),
            ),
            cast_to=GeofenceContainsResponse,
        )


class AsyncGeofenceResource(AsyncAPIResource):
    @cached_property
    def console(self) -> AsyncConsoleResource:
        return AsyncConsoleResource(self._client)

    @cached_property
    def batch(self) -> AsyncBatchResource:
        return AsyncBatchResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncGeofenceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncGeofenceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGeofenceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncGeofenceResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        key: str,
        type: Literal["`circle`", "`polygon`", "`isochrone`"],
        circle: geofence_create_params.Circle | NotGiven = NOT_GIVEN,
        custom_id: str | NotGiven = NOT_GIVEN,
        isochrone: geofence_create_params.Isochrone | NotGiven = NOT_GIVEN,
        meta_data: object | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        polygon: geofence_create_params.Polygon | NotGiven = NOT_GIVEN,
        tags: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GeofenceCreateResponse:
        """
        Create a geofence

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          type: Specify the type of the geofence that is being created.

          circle: Provide the details to create a circular geofence. Please note that this object
              is mandatory when `type` is `circle`. When the `type` is not `circle`, the
              properties of this object will be ignored while creating the geofence.

          custom_id: Set an unique ID for the new geofence. If not provided, an ID will be
              automatically generated in UUID format. A valid `custom_id` can contain letters,
              numbers, "-", & "\\__" only.

              Please note that the ID of a geofence can not be changed once it is created.

          isochrone: Provide the details to create an isochrone based geofence. Use this object when
              `type` is `isochrone`. When the `type` is not `isochrone`, the properties of
              this object will be ignored while creating the geofence.

          meta_data: Metadata of the geofence. Use this field to define custom attributes that
              provide more context and information about the geofence being created like
              country, group ID etc.

              The data being added should be in valid JSON object format (i.e. `key` and
              `value` pairs). Max size allowed for the object is 65kb.

          name: Name of the geofence. Use this field to assign a meaningful, custom name to the
              geofence being created.

          polygon: Provide the details to create a custom polygon type of geofence. Please note
              that this object is mandatory when `type` is `polygon`. When the `type` is not
              `polygon`, the properties of this object will be ignored while creating the
              geofence.

              Self-intersecting polygons or polygons containing other polygons are invalid and
              will be removed while processing the request.

              Area of the polygon should be less than 2000 km<sup>2</sup>.

          tags: An array of strings to associate multiple tags to the geofence. `tags` can be
              used to search or filter geofences (using `Get Geofence List` method).

              Create valid `tags` using a string consisting of alphanumeric characters (A-Z,
              a-z, 0-9) along with the underscore ('\\__') and hyphen ('-') symbols.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/geofence",
            body=await async_maybe_transform(
                {
                    "type": type,
                    "circle": circle,
                    "custom_id": custom_id,
                    "isochrone": isochrone,
                    "meta_data": meta_data,
                    "name": name,
                    "polygon": polygon,
                    "tags": tags,
                },
                geofence_create_params.GeofenceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, geofence_create_params.GeofenceCreateParams),
            ),
            cast_to=GeofenceCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GeofenceRetrieveResponse:
        """
        Get a Geofence

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/geofence/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, geofence_retrieve_params.GeofenceRetrieveParams),
            ),
            cast_to=GeofenceRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        key: str,
        circle: geofence_update_params.Circle | NotGiven = NOT_GIVEN,
        isochrone: geofence_update_params.Isochrone | NotGiven = NOT_GIVEN,
        meta_data: object | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        polygon: geofence_update_params.Polygon | NotGiven = NOT_GIVEN,
        tags: List[str] | NotGiven = NOT_GIVEN,
        type: Literal["`circle`", "`polygon`", "`isochrone`"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Update a Geofence

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          circle: Use this object to update details of a circular geofence. Please note that this
              object is mandatory only when `type` is `circle`. When the `type` is not
              `circle`, the properties of this object will be ignored while creating the
              geofence.

          isochrone: Use this object to update details of an isochrone based geofence. Please note
              that this object is mandatory only when `type` is `isochrone`. When the `type`
              is not `isochrone`, the properties of this object will be ignored while creating
              the geofence.

          meta_data: Updated the `meta_data` associated with a geofence. Use this field to define
              custom attributes that provide more context and information about the geofence
              being updated like country, group ID etc.

              The data being added should be in valid JSON object format (i.e. `key` and
              `value` pairs). Max size allowed for the object is 65kb.

          name: Use this parameter to update the `name` of a geofence. Users can assign
              meaningful custom names to their geofences.

          polygon: Use this object to update details of a custom polygon geofence. Please note that
              this object is mandatory only when `type` is `polygon`. When the `type` is not
              `polygon`, the properties of this object will be ignored while creating the
              geofence.

              Self-intersecting polygons or polygons containing other polygons are invalid and
              will be removed while processing the request.

              Area of the polygon should be less than 2000 km<sup>2</sup>.

          tags: Use this parameter to add/modify one or multiple `tags` of a geofence. `tags`
              can be used to search or filter geofences (using `Get Geofence List` method).

              Valid values for updating `tags` consist of alphanumeric characters (A-Z, a-z,
              0-9) along with the underscore ('\\__') and hyphen ('-') symbols.

          type: Use this parameter to update the `type` of a geofence. Please note that you will
              need to provide required details for creating a geofence of the new `type`.
              Check other parameters of this method to know more.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/geofence/{id}",
            body=await async_maybe_transform(
                {
                    "circle": circle,
                    "isochrone": isochrone,
                    "meta_data": meta_data,
                    "name": name,
                    "polygon": polygon,
                    "tags": tags,
                    "type": type,
                },
                geofence_update_params.GeofenceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, geofence_update_params.GeofenceUpdateParams),
            ),
            cast_to=SimpleResp,
        )

    async def list(
        self,
        *,
        key: str,
        pn: int | NotGiven = NOT_GIVEN,
        ps: int | NotGiven = NOT_GIVEN,
        tags: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GeofenceListResponse:
        """
        Get Geofence List

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          pn: Denotes page number. Use this along with the `ps` parameter to implement
              pagination for your searched results. This parameter does not have a maximum
              limit but would return an empty response in case a higher value is provided when
              the result-set itself is smaller.

          ps: Denotes number of search results per page. Use this along with the `pn`
              parameter to implement pagination for your searched results.

          tags: Comma (`,`) separated list of `tags` which will be used to filter the geofences.

              Please note only the geofences which have all the `tags` added to this parameter
              will be included in the result. This parameter can accept a string with a
              maximum length of 256 characters.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/geofence/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "pn": pn,
                        "ps": ps,
                        "tags": tags,
                    },
                    geofence_list_params.GeofenceListParams,
                ),
            ),
            cast_to=GeofenceListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Delete a Geofence

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/geofence/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, geofence_delete_params.GeofenceDeleteParams),
            ),
            cast_to=SimpleResp,
        )

    async def contains(
        self,
        *,
        key: str,
        locations: str,
        geofences: str | NotGiven = NOT_GIVEN,
        verbose: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GeofenceContainsResponse:
        """
        Geofence Contains

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          locations: Pipe (|) separated coordinates, in [latitude,longitude] format, of the locations
              to be searched against the geofences.

          geofences: A `,` separated list geofence IDs against which the `locations` will be
              searched. If not provided, then the 'locations' will be searched against all
              your existing geofences.

              Maximum length of the string can be 256 characters.

          verbose: When `true`, an array with detailed information of geofences is returned. When
              `false`, an array containing only the IDs of the geofences is returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/geofence/contain",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "locations": locations,
                        "geofences": geofences,
                        "verbose": verbose,
                    },
                    geofence_contains_params.GeofenceContainsParams,
                ),
            ),
            cast_to=GeofenceContainsResponse,
        )


class GeofenceResourceWithRawResponse:
    def __init__(self, geofence: GeofenceResource) -> None:
        self._geofence = geofence

        self.create = to_raw_response_wrapper(
            geofence.create,
        )
        self.retrieve = to_raw_response_wrapper(
            geofence.retrieve,
        )
        self.update = to_raw_response_wrapper(
            geofence.update,
        )
        self.list = to_raw_response_wrapper(
            geofence.list,
        )
        self.delete = to_raw_response_wrapper(
            geofence.delete,
        )
        self.contains = to_raw_response_wrapper(
            geofence.contains,
        )

    @cached_property
    def console(self) -> ConsoleResourceWithRawResponse:
        return ConsoleResourceWithRawResponse(self._geofence.console)

    @cached_property
    def batch(self) -> BatchResourceWithRawResponse:
        return BatchResourceWithRawResponse(self._geofence.batch)


class AsyncGeofenceResourceWithRawResponse:
    def __init__(self, geofence: AsyncGeofenceResource) -> None:
        self._geofence = geofence

        self.create = async_to_raw_response_wrapper(
            geofence.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            geofence.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            geofence.update,
        )
        self.list = async_to_raw_response_wrapper(
            geofence.list,
        )
        self.delete = async_to_raw_response_wrapper(
            geofence.delete,
        )
        self.contains = async_to_raw_response_wrapper(
            geofence.contains,
        )

    @cached_property
    def console(self) -> AsyncConsoleResourceWithRawResponse:
        return AsyncConsoleResourceWithRawResponse(self._geofence.console)

    @cached_property
    def batch(self) -> AsyncBatchResourceWithRawResponse:
        return AsyncBatchResourceWithRawResponse(self._geofence.batch)


class GeofenceResourceWithStreamingResponse:
    def __init__(self, geofence: GeofenceResource) -> None:
        self._geofence = geofence

        self.create = to_streamed_response_wrapper(
            geofence.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            geofence.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            geofence.update,
        )
        self.list = to_streamed_response_wrapper(
            geofence.list,
        )
        self.delete = to_streamed_response_wrapper(
            geofence.delete,
        )
        self.contains = to_streamed_response_wrapper(
            geofence.contains,
        )

    @cached_property
    def console(self) -> ConsoleResourceWithStreamingResponse:
        return ConsoleResourceWithStreamingResponse(self._geofence.console)

    @cached_property
    def batch(self) -> BatchResourceWithStreamingResponse:
        return BatchResourceWithStreamingResponse(self._geofence.batch)


class AsyncGeofenceResourceWithStreamingResponse:
    def __init__(self, geofence: AsyncGeofenceResource) -> None:
        self._geofence = geofence

        self.create = async_to_streamed_response_wrapper(
            geofence.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            geofence.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            geofence.update,
        )
        self.list = async_to_streamed_response_wrapper(
            geofence.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            geofence.delete,
        )
        self.contains = async_to_streamed_response_wrapper(
            geofence.contains,
        )

    @cached_property
    def console(self) -> AsyncConsoleResourceWithStreamingResponse:
        return AsyncConsoleResourceWithStreamingResponse(self._geofence.console)

    @cached_property
    def batch(self) -> AsyncBatchResourceWithStreamingResponse:
        return AsyncBatchResourceWithStreamingResponse(self._geofence.batch)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["PlaceItem", "Geopoint", "Poi"]


class Geopoint(BaseModel):
    lat: Optional[float] = None
    """This parameter represents the latitude value of the place."""

    lng: Optional[float] = None
    """This parameter represents the longitude value of the place."""


class Poi(BaseModel):
    title: Optional[str] = None
    """A title that describes the point of interest."""


class PlaceItem(BaseModel):
    address: Optional[str] = None
    """
    This parameter represents the complete address of the place, including the
    street, city, state, postal code and country.
    """

    building: Optional[str] = None
    """This parameter represents additional building information if applicable."""

    city: Optional[str] = None
    """This parameter represents the city or town of the place."""

    country: Optional[str] = None
    """This parameter represents the country of the place."""

    district: Optional[str] = None
    """This parameter represents the district of the place."""

    geopoint: Optional[Geopoint] = None
    """This parameter represents the geographical coordinates of the place.

    It includes the latitude and longitude values.
    """

    house: Optional[str] = None
    """This parameter represents the house or building number of the place."""

    poi: Optional[Poi] = None
    """This parameter represents a point of interest within the place.

    A Point of Interest (POI) refers to a specific location or area that is of
    interest to individuals for various reasons. It could be a landmark, tourist
    attraction, business, or any other location that people might find important or
    intriguing.
    """

    postal_code: Optional[str] = FieldInfo(alias="postalCode", default=None)
    """This parameter represents the postal code or ZIP code of the place."""

    state: Optional[str] = None
    """This parameter represents the state or region of the place."""

    street: Optional[str] = None
    """This parameter represents the street name of the place."""

    sub_district: Optional[str] = FieldInfo(alias="subDistrict", default=None)
    """This parameter represents the sub-district or locality of the place."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.skynet.asset import event_list_params
from ....types.skynet.asset.event_list_response import EventListResponse

__all__ = ["EventResource", "AsyncEventResource"]


class EventResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EventResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return EventResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EventResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return EventResourceWithStreamingResponse(self)

    def list(
        self,
        id: str,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        end_time: int | NotGiven = NOT_GIVEN,
        monitor_id: str | NotGiven = NOT_GIVEN,
        pn: int | NotGiven = NOT_GIVEN,
        ps: int | NotGiven = NOT_GIVEN,
        start_time: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EventListResponse:
        """
        Event History of an Asset

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          end_time: Time before which the events triggered by the `asset` need to be retrieved.

          monitor_id: Filter the events by `monitor_id`. When provided, only the events triggered by
              the `monitor` will be returned in response.

              Please note that if the `attributes` of the asset identified by `id` and those
              of the `monitor` do not match, then no events might be returned for this
              `monitor_id`.

          pn: Denotes page number. Use this along with the `ps` parameter to implement
              pagination for your searched results. This parameter does not have a maximum
              limit but would return an empty response in case a higher value is provided when
              the result-set itself is smaller.

          ps: Denotes number of search results per page. Use this along with the `pn`
              parameter to implement pagination for your searched results.

          start_time: Time after which the events triggered by the `asset` need to be retrieved.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/skynet/asset/{id}/event/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                        "end_time": end_time,
                        "monitor_id": monitor_id,
                        "pn": pn,
                        "ps": ps,
                        "start_time": start_time,
                    },
                    event_list_params.EventListParams,
                ),
            ),
            cast_to=EventListResponse,
        )


class AsyncEventResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEventResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEventResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEventResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncEventResourceWithStreamingResponse(self)

    async def list(
        self,
        id: str,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        end_time: int | NotGiven = NOT_GIVEN,
        monitor_id: str | NotGiven = NOT_GIVEN,
        pn: int | NotGiven = NOT_GIVEN,
        ps: int | NotGiven = NOT_GIVEN,
        start_time: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EventListResponse:
        """
        Event History of an Asset

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          end_time: Time before which the events triggered by the `asset` need to be retrieved.

          monitor_id: Filter the events by `monitor_id`. When provided, only the events triggered by
              the `monitor` will be returned in response.

              Please note that if the `attributes` of the asset identified by `id` and those
              of the `monitor` do not match, then no events might be returned for this
              `monitor_id`.

          pn: Denotes page number. Use this along with the `ps` parameter to implement
              pagination for your searched results. This parameter does not have a maximum
              limit but would return an empty response in case a higher value is provided when
              the result-set itself is smaller.

          ps: Denotes number of search results per page. Use this along with the `pn`
              parameter to implement pagination for your searched results.

          start_time: Time after which the events triggered by the `asset` need to be retrieved.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/skynet/asset/{id}/event/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                        "end_time": end_time,
                        "monitor_id": monitor_id,
                        "pn": pn,
                        "ps": ps,
                        "start_time": start_time,
                    },
                    event_list_params.EventListParams,
                ),
            ),
            cast_to=EventListResponse,
        )


class EventResourceWithRawResponse:
    def __init__(self, event: EventResource) -> None:
        self._event = event

        self.list = to_raw_response_wrapper(
            event.list,
        )


class AsyncEventResourceWithRawResponse:
    def __init__(self, event: AsyncEventResource) -> None:
        self._event = event

        self.list = async_to_raw_response_wrapper(
            event.list,
        )


class EventResourceWithStreamingResponse:
    def __init__(self, event: EventResource) -> None:
        self._event = event

        self.list = to_streamed_response_wrapper(
            event.list,
        )


class AsyncEventResourceWithStreamingResponse:
    def __init__(self, event: AsyncEventResource) -> None:
        self._event = event

        self.list = async_to_streamed_response_wrapper(
            event.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import browse_search_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.browse_search_response import BrowseSearchResponse

__all__ = ["BrowseResource", "AsyncBrowseResource"]


class BrowseResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BrowseResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return BrowseResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BrowseResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return BrowseResourceWithStreamingResponse(self)

    def search(
        self,
        *,
        key: str,
        at: str | NotGiven = NOT_GIVEN,
        categories: str | NotGiven = NOT_GIVEN,
        in_: str | NotGiven = NOT_GIVEN,
        lang: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BrowseSearchResponse:
        """
        Browse a search area using a free text query

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          at: Specify the center of the search context expressed as coordinates.

              Please note that one of "at", "in=circle" or "in=bbox" should be provided for
              relevant results.

          categories: This is a category filter consisting of a comma-separated list of categories.
              Places with any assigned categories that match any of the requested categories
              are included in the response.

          in_: Search within a geographic area. This is a hard filter. Results will be returned
              if they are located within the specified area.

              A geographic area can be

              - a country (or multiple countries), provided as comma-separated
                [ISO 3166-1 alpha-3](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-3) country
                codes

                The country codes are to be provided in all uppercase.

                Format: `countryCode:{countryCode}[,{countryCode}]`

              - a circular area, provided as latitude, longitude, and radius (an integer with
                meters as unit)

                Format: `circle:{latitude},{longitude};r={radius}`

              - a bounding box, provided as _west longitude_, _south latitude_, _east
                longitude_, _north latitude_

                Format:
                `bbox:{west longitude},{south latitude},{east longitude},{north latitude}`

              Please provide one of 'at', 'in=circle' or 'in=bbox' input for a relevant
              result.

          lang: Select the language to be used for result rendering from a list of
              [BCP 47](https://en.wikipedia.org/wiki/IETF_language_tag) compliant language
              codes.

          limit: Sets the maximum number of results to be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/browse",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "at": at,
                        "categories": categories,
                        "in_": in_,
                        "lang": lang,
                        "limit": limit,
                    },
                    browse_search_params.BrowseSearchParams,
                ),
            ),
            cast_to=BrowseSearchResponse,
        )


class AsyncBrowseResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBrowseResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBrowseResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBrowseResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncBrowseResourceWithStreamingResponse(self)

    async def search(
        self,
        *,
        key: str,
        at: str | NotGiven = NOT_GIVEN,
        categories: str | NotGiven = NOT_GIVEN,
        in_: str | NotGiven = NOT_GIVEN,
        lang: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BrowseSearchResponse:
        """
        Browse a search area using a free text query

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          at: Specify the center of the search context expressed as coordinates.

              Please note that one of "at", "in=circle" or "in=bbox" should be provided for
              relevant results.

          categories: This is a category filter consisting of a comma-separated list of categories.
              Places with any assigned categories that match any of the requested categories
              are included in the response.

          in_: Search within a geographic area. This is a hard filter. Results will be returned
              if they are located within the specified area.

              A geographic area can be

              - a country (or multiple countries), provided as comma-separated
                [ISO 3166-1 alpha-3](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-3) country
                codes

                The country codes are to be provided in all uppercase.

                Format: `countryCode:{countryCode}[,{countryCode}]`

              - a circular area, provided as latitude, longitude, and radius (an integer with
                meters as unit)

                Format: `circle:{latitude},{longitude};r={radius}`

              - a bounding box, provided as _west longitude_, _south latitude_, _east
                longitude_, _north latitude_

                Format:
                `bbox:{west longitude},{south latitude},{east longitude},{north latitude}`

              Please provide one of 'at', 'in=circle' or 'in=bbox' input for a relevant
              result.

          lang: Select the language to be used for result rendering from a list of
              [BCP 47](https://en.wikipedia.org/wiki/IETF_language_tag) compliant language
              codes.

          limit: Sets the maximum number of results to be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/browse",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "at": at,
                        "categories": categories,
                        "in_": in_,
                        "lang": lang,
                        "limit": limit,
                    },
                    browse_search_params.BrowseSearchParams,
                ),
            ),
            cast_to=BrowseSearchResponse,
        )


class BrowseResourceWithRawResponse:
    def __init__(self, browse: BrowseResource) -> None:
        self._browse = browse

        self.search = to_raw_response_wrapper(
            browse.search,
        )


class AsyncBrowseResourceWithRawResponse:
    def __init__(self, browse: AsyncBrowseResource) -> None:
        self._browse = browse

        self.search = async_to_raw_response_wrapper(
            browse.search,
        )


class BrowseResourceWithStreamingResponse:
    def __init__(self, browse: BrowseResource) -> None:
        self._browse = browse

        self.search = to_streamed_response_wrapper(
            browse.search,
        )


class AsyncBrowseResourceWithStreamingResponse:
    def __init__(self, browse: AsyncBrowseResource) -> None:
        self._browse = browse

        self.search = async_to_streamed_response_wrapper(
            browse.search,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types import RevgeocodeRetrieveResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestRevgeocode:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: NextbillionSDK) -> None:
        revgeocode = client.revgeocode.retrieve(
            at="at=52.5308,13.3856",
            key="key=API_KEY",
        )
        assert_matches_type(RevgeocodeRetrieveResponse, revgeocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_with_all_params(self, client: NextbillionSDK) -> None:
        revgeocode = client.revgeocode.retrieve(
            at="at=52.5308,13.3856",
            key="key=API_KEY",
            in_="in=countryCode:CAN,MEX,USA",
            lang="lang=en",
        )
        assert_matches_type(RevgeocodeRetrieveResponse, revgeocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: NextbillionSDK) -> None:
        response = client.revgeocode.with_raw_response.retrieve(
            at="at=52.5308,13.3856",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        revgeocode = response.parse()
        assert_matches_type(RevgeocodeRetrieveResponse, revgeocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: NextbillionSDK) -> None:
        with client.revgeocode.with_streaming_response.retrieve(
            at="at=52.5308,13.3856",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            revgeocode = response.parse()
            assert_matches_type(RevgeocodeRetrieveResponse, revgeocode, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncRevgeocode:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        revgeocode = await async_client.revgeocode.retrieve(
            at="at=52.5308,13.3856",
            key="key=API_KEY",
        )
        assert_matches_type(RevgeocodeRetrieveResponse, revgeocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        revgeocode = await async_client.revgeocode.retrieve(
            at="at=52.5308,13.3856",
            key="key=API_KEY",
            in_="in=countryCode:CAN,MEX,USA",
            lang="lang=en",
        )
        assert_matches_type(RevgeocodeRetrieveResponse, revgeocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.revgeocode.with_raw_response.retrieve(
            at="at=52.5308,13.3856",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        revgeocode = await response.parse()
        assert_matches_type(RevgeocodeRetrieveResponse, revgeocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.revgeocode.with_streaming_response.retrieve(
            at="at=52.5308,13.3856",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            revgeocode = await response.parse()
            assert_matches_type(RevgeocodeRetrieveResponse, revgeocode, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .polygon_get_params import PolygonGetParams as PolygonGetParams
from .polygon_create_params import PolygonCreateParams as PolygonCreateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

from .vehicle_param import VehicleParam

__all__ = [
    "DriverAssignmentAssignParams",
    "Filter",
    "Order",
    "OrderPickup",
    "OrderDropoff",
    "OrderVehiclePreferences",
    "OrderVehiclePreferencesExcludeAllOfAttribute",
    "OrderVehiclePreferencesRequiredAllOfAttribute",
    "OrderVehiclePreferencesRequiredAnyOfAttribute",
    "Options",
    "OptionsOrderAttributePriorityMapping",
    "OptionsVehicleAttributePriorityMapping",
]


class DriverAssignmentAssignParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    filter: Required[Filter]
    """
    Specify the filtering criterion for the vehicles with respect to each order's
    location. `filter` is a mandatory input for all requests.
    """

    orders: Required[Iterable[Order]]
    """Collects the details of open orders to be fulfilled.

    Each object represents one order. All requests must include `orders` as a
    mandatory input. A maximum of 200 orders is allowed per request.
    """

    vehicles: Required[Iterable[VehicleParam]]
    """Collects the details of vehicles available to fulfill the orders.

    Each object represents one vehicle. All requests must include `vehicles` as a
    mandatory input. A maximum of 100 vehicles is allowed per request.
    """

    options: Options
    """Configure the assignment constraints and response settings."""


class Filter(TypedDict, total=False):
    driving_distance: float
    """Defines a `driving_distance` filter, in meters.

    If a vehicle needs to drive further than this distance to reach a pickup
    location, it will not be assigned to that order. Valid range of values for this
    filter is \\[[1, 10000\\]].
    """

    pickup_eta: int
    """
    Specify a duration, in seconds, which will be used to filter out ineligible
    vehicles for each order. Any vehicle which would take more time than specified
    here, to reach the pickup location of a given order, will be ruled out for
    assignment for that particular order. Valid values for `pickup_eta` are \\[[1,
    3600\\]].
    """

    radius: float
    """
    Specify a radius, in meters, which will be used to filter out ineligible
    vehicles for each order. The pickup location of an order will act as the center
    of the circle when identifying eligible vehicles. Valid values for `radius` are
    \\[[1, 10000\\]].
    """


class OrderPickup(TypedDict, total=False):
    lat: float
    """Latitude of the pickup location."""

    lng: float
    """Longitude of the pickup location."""


class OrderDropoff(TypedDict, total=False):
    lat: float
    """Latitude of the stop location."""

    lng: float
    """Longitude of the stop location."""


class OrderVehiclePreferencesExcludeAllOfAttribute(TypedDict, total=False):
    attribute: Required[str]
    """Specify the name of the attribute.

    The `attribute` is compared to the keys (of each `key:value` pair) in
    `vehicles.attributes` during evaluation.
    """

    operator: Required[str]
    """
    Specify the operator to denote the relation between `attribute` and the `value`
    specified above. The `attribute` , `operator` and `value` together constitute
    the condition that a vehicle must meet to be eligible for assignment. Currently,
    we support following operators currently:

    - Equal to (==)
    - Less than (<)
    - Less tha equal to (<=)
    - Greater than (>)
    - Greater than equal to (>=)
    - Contains (contains)

    Please note that when using "contains" operator only one `value` can be
    specified and the corresponding `attribute` must contain multiple values when
    defined for a vehicle.
    """

    value: Required[str]
    """Specify the desired value of the attribute to be applied for this order.

    `value` provided here is compared to the values (of each `key:value` pair) in
    `vehicles.attributes` during evaluation.
    """


class OrderVehiclePreferencesRequiredAllOfAttribute(TypedDict, total=False):
    attribute: Required[str]
    """Specify the name of the attribute.

    The `attribute` is compared to the keys (of each `key:value` pair) in
    `vehicles.attributes` during evaluation.
    """

    operator: Required[str]
    """
    Specify the operator to denote the relation between `attribute` and the `value`
    specified above. The `attribute` , `operator` and `value` together constitute
    the condition that a vehicle must meet to be eligible for assignment. Currently,
    we support following operators currently:

    - Equal to (==)
    - Less than (<)
    - Less tha equal to (<=)
    - Greater than (>)
    - Greater than equal to (>=)
    - Contains (contains)

    Please note that when using "contains" operator only one `value` can be
    specified and the corresponding `attribute` must contain multiple values when
    defined for a vehicle.
    """

    value: Required[str]
    """Specify the desired value of the attribute to be applied for this order.

    `value` provided here is compared to the values (of each `key:value` pair) in
    `vehicles.attributes` during evaluation.
    """


class OrderVehiclePreferencesRequiredAnyOfAttribute(TypedDict, total=False):
    attribute: Required[str]
    """Specify the name of the attribute.

    The `attribute` is compared to the keys (of each `key:value` pair) in
    `vehicles.attributes` during evaluation.
    """

    operator: Required[str]
    """
    Specify the operator to denote the relation between `attribute` and the `value`
    specified above. The `attribute` , `operator` and `value` together constitute
    the condition that a vehicle must meet to be eligible for assignment. Currently,
    we support following operators currently:

    - Equal to (==)
    - Less than (<)
    - Less tha equal to (<=)
    - Greater than (>)
    - Greater than equal to (>=)
    - Contains (contains)

    Please note that when using "contains" operator only one `value` can be
    specified and the corresponding `attribute` must contain multiple values when
    defined for a vehicle.
    """

    value: Required[str]
    """Specify the desired value of the attribute to be applied for this order.

    `value` provided here is compared to the values (of each `key:value` pair) in
    `vehicles.attributes` during evaluation.
    """


class OrderVehiclePreferences(TypedDict, total=False):
    exclude_all_of_attributes: Iterable[OrderVehiclePreferencesExcludeAllOfAttribute]
    """An array of objects to add exclusion requirements for the order.

    A vehicle must **not meet any of the conditions** specified here to be
    considered for assignment. Each object represents a single condition. Please
    note that a maximum of 10 conditions can be added here for a given order.
    """

    required_all_of_attributes: Iterable[OrderVehiclePreferencesRequiredAllOfAttribute]
    """An array of objects to add mandatory requirements for the order.

    A vehicle must **meet** **all conditions** specified here to be considered for
    assignment. Each object represents a single condition. Please note that a
    maximum of 10 conditions can be added here for a given order.
    """

    required_any_of_attributes: Iterable[OrderVehiclePreferencesRequiredAnyOfAttribute]
    """An array of objects to add optional requirements for the order.

    A vehicle must **meet** **at least one of the conditions** specified here to be
    considered for assignment. Each object represents a single condition. Please
    note that a maximum of 10 conditions can be added here for a given order.
    """


class Order(TypedDict, total=False):
    id: Required[str]
    """Specify a unique ID for the order."""

    pickup: Required[OrderPickup]
    """Specify the location coordinates of the pickup location of the order.

    This input is mandatory for each order.
    """

    attributes: object
    """Specify custom attributes for the orders.

    Each attribute should be created as a `key:value` pair. The **keys** provided
    can be used in `options.order_attribute_priority_mappings` to assign a custom
    priority for this order based on its attributes.

    The maximum number of key:value pairs that can be specified under `attributes`
    for a given order, is limited to 30.
    """

    dropoffs: Iterable[OrderDropoff]
    """
    Use this parameter to specify the location coordinates of the destination of the
    trip or the intermediate stops to be completed before it.

    Please note

    - The last location provided is treated as the destination of the trip.
    - `dropoffs` is mandatory when `dropoff_details` is set to **true**.
    """

    priority: int
    """Specify the priority for this order.

    A higher value indicates a higher priority. When specified, it will override any
    priority score deduced from `order_attribute_priority_mappings` for this order.
    Valid values are \\[[1, 10\\]] and default is 0.
    """

    service_time: int
    """Specify the service time, in seconds, for the order.

    Service time is the duration that the driver is likely to wait at the pickup
    location after arriving. The impact of the service time is realized in the ETA
    for the "dropoff" type step.
    """

    vehicle_preferences: OrderVehiclePreferences
    """Define custom preferences for task assignment based on vehicle's attributes.

    If multiple criteria are provided, they are evaluated using an AND
    condition—meaning all specified criteria must be met individually for a vehicle
    to be considered.

    For example, if `required_all_of_attributes`, `required_any_of_attributes`, and
    `exclude_all_of_attributes` are all provided, an eligible vehicle must satisfy
    the following to be considered for assignments:

    1.  Meet all conditions specified in `required_all_of_attributes`.
    2.  Meet at least one of the conditions listed in `required_any_of_attributes`.
    3.  Not meet any conditions mentioned in `exclude_all_of_attributes`.

    Consequently, a vehicle which does not have any `attributes` defined can't be
    assigned to an order which has `vehicle_preferences` configured.
    """


class OptionsOrderAttributePriorityMapping(TypedDict, total=False):
    attribute: Required[str]
    """Specify the name of the attribute.

    The `attribute` is compared to the keys (of each `key:value` pair) in
    `orders.attributes` during evaluation.
    """

    operator: Required[str]
    """
    Specify the operator to denote the relation between `attribute` and the `value`
    specified above. The `attribute` , `operator` and `value` together constitute
    the condition that an order must meet to assume the specified priority. We
    support the following operators currently:

    - Equal to (==)
    - Less than (<)
    - Less tha equal to (<=)
    - Greater than (>)
    - Greater than equal to (>=)
    - Contains (contains)

    Please note that when using "contains" operator only one `value` can be
    specified and the corresponding `attribute` must contain multiple values when
    defined for an order.
    """

    priority: Required[str]
    """
    Specify the priority score that should be assigned when an order qualifies the
    criteria specified above. A higher value indicates a higher priority. Valid
    values are \\[[1,10\\]].
    """

    value: Required[str]
    """Specify the desired value of the attribute to be applied for this order.

    `value` provided here is compared to the values (of each `key:value` pair) in
    `orders.attributes` during evaluation.
    """


class OptionsVehicleAttributePriorityMapping(TypedDict, total=False):
    attribute: Required[str]
    """Specify the name of the attribute.

    The `attribute` is compared to the keys (of each `key:value` pair) in
    `vehicles.attributes` during evaluation.
    """

    operator: Required[str]
    """
    Specify the operator to denote the relation between `attribute` and the `value`
    specified above. The `attribute` , `operator` and `value` together constitute
    the condition that a vehicle must meet to assume the specified priority. We
    support the following operators currently:

    - Equal to (==)
    - Less than (<)
    - Less tha equal to (<=)
    - Greater than (>)
    - Greater than equal to (>=)
    - Contains (contains)

    Please note that when using "contains" operator only one `value` can be
    specified and the corresponding `attribute` must contain multiple values when
    defined for a vehicle.
    """

    priority: Required[str]
    """
    Specify the priority score that should be assigned when a vehicle qualifies the
    criteria specified above. A higher value indicates a higher priority. Valid
    values are \\[[1,10\\]].
    """

    value: Required[str]
    """Specify the desired value of the attribute to be applied for this vehicle.

    `value` provided here is compared to the values (of each `key:value` pair) in
    `vehicles.attributes` during evaluation.
    """


class Options(TypedDict, total=False):
    alternate_assignments: int
    """
    Specify the maximum number of potential, alternate vehicle assignments to be
    returned for each order, apart from the vehicle which was assigned as
    recommended. Please note that:

    - The maximum number of alternate assignments that can be requested are 3.
    - It is not necessary that the service will return the specified number of
      alternate assignments for each order. The number of alternate assignments
      returned will depend on the number of vehicles provided in the input.
    - Order which could not be assigned to any vehicles due to their `filter` or
      attribute matching criteria will not be eligible for alternate assignments as
      well.
    """

    dropoff_details: bool
    """
    When **true**, the service returns the drop-off steps for each trip and related
    details in the response. Defaults to **false**.
    """

    order_attribute_priority_mappings: Iterable[OptionsOrderAttributePriorityMapping]
    """
    Collection of rules for assigning custom priority to orders based on their
    attributes. In case an order satisfies more than one rule, the highest priority
    score from all the rules satisfied, would be the effective priority score for
    such an order.
    """

    travel_cost: Literal["driving_eta", "driving_distance", "straight_line_distance"]
    """
    Choose a travel cost that will be used by the service for assigning vehicles
    efficiently from a set of qualifying ones.
    """

    vehicle_attribute_priority_mappings: Iterable[OptionsVehicleAttributePriorityMapping]
    """
    Collection of rules for assigning custom priority to vehicles based on their
    attributes. In case a vehicle satisfies more than one rule, the highest priority
    score from all the rules satisfied, would be the effective priority score for
    such a vehicle.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["GeofenceContainsParams"]


class GeofenceContainsParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    locations: Required[str]
    """
    Pipe (|) separated coordinates, in [latitude,longitude] format, of the locations
    to be searched against the geofences.
    """

    geofences: str
    """A `,` separated list geofence IDs against which the `locations` will be
    searched.

    If not provided, then the 'locations' will be searched against all your existing
    geofences.

    Maximum length of the string can be 256 characters.
    """

    verbose: str
    """When `true`, an array with detailed information of geofences is returned.

    When `false`, an array containing only the IDs of the geofences is returned.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["DirectionComputeRouteParams"]


class DirectionComputeRouteParams(TypedDict, total=False):
    destination: Required[str]

    origin: Required[str]

    altcount: int
    """Sets the number of alternative routes to return.

    It is effective only when alternatives=true. Default to 3.

    Please note that adding alternative route count does not guarantee matching
    number of routes to be returned if potential alternative routes do not exist.
    """

    alternatives: bool
    """When true the API will return alternate routes.

    The alternatives is effective only when there are no waypoints included in the
    request.

    You can set the number of alternate routes to be returned in the altcount
    property.
    """

    approaches: str
    """
    A semicolon-separated list indicating the side of the road from which to
    approach waypoints in a requested route.

    When set to unrestricted a route can arrive at the waypoint from either side of
    the road and when set to curb the route will arrive at the waypoint on the
    driving side of the region.

    Please note the number of values provided must be one more than the number of
    waypoints. The last value of approaches will determine the approach for the
    destination. However, you can skip a coordinate and show its position in the
    list with the ; separator.
    """

    avoid: Literal[
        "toll",
        "ferry",
        "highway",
        "none",
        "sharp_turn",
        "uturn",
        "service_road",
        "left_turn",
        "right_turn",
        "bbox",
        "geofence_id",
        "tunnel",
    ]
    """When option=fast (by default):

    Setting this will ensure the route avoids ferries, tolls, highways or nothing.
    Multiple values should be separated by a pipe "|". If none is provided along
    with other values, an error is returned as a valid route is not feasible.

    Please note that when this parameter is not provided in the input, ferries are
    set to be avoided by default. When this parameter is provided, only the
    mentioned objects are avoided.

    When option=flexible:

    Set this parameter to find alternative routes that bypass specified objects. Use
    a pipe "|" to separate multiple values. This is a flexible filter; if no
    alternative routes exist, the service will still provide a route that includes
    the objects. For a strict filter, consider using the exclude parameter.

    \\-- This parameter is effective only when route_type=fastest.
    \\-- Following objects are exceptions to the flexible filtering behavior of avoid
    parameter: bbox, tunnel and geofence_id. When used, the service will return an
    error in case there are no alternative routes available.
    \\-- When using avoid=bbox users also need to specify the boundaries of the
    bounding box to be avoided. Multiple bounding boxes can be specified
    simultaneously. The perimeter of a bounding box can not exceed 500 KM. Format:
    bbox=min_latitude,min_longtitude,max_latitude,max_longitude. Example:
    avoid=bbox: 34.0635,-118.2547, 34.0679,-118.2478 | bbox: 34.0521,-118.2342,
    34.0478,-118.2437
    \\-- When using avoid=sharp_turn, default range of permissible turn angles is
    \\[[120,240\\]] in the clockwise direction from the current road. In order to
    override default range, please use turn_angle_range parameter.
    \\-- When using avoid=geofence_id, only the the geofences created using
    [NextBillion.ai](http://NextBillion.ai) Geofence API are valid.
    \\-- When this parameter is not provided in the input, ferry routes are set to be
    avoided by default. When this parameter is provided, only the mentioned
    object(s) are avoided.
    \\-- If none is provided along with other values, an error is returned as a valid
    route is not feasible.
    """

    bearings: str
    """
    Limits the search to road segments with given bearing, in degrees, towards true
    north in clockwise direction. Each bearings should be in the format of
    degree,range, where the degree should be a value between \\[[0, 360\\]] and range
    should be a value between \\[[0, 180\\]].

    Please note that the number of bearings should be two more than the number of
    waypoints. This is to account for the bearing of origin and destination. If a
    route can approach a waypoint or the destination from any direction, the bearing
    for that point can be specified as "0,180".
    """

    cross_border: bool
    """Requires option=flexible.

    Specify if crossing an international border is expected for operations near
    border areas. When set to false, the API will prohibit routes crossing the
    borders. When set to true, the service will return routes which cross the
    borders between countries, if required for the given set destination and
    waypoints.

    This feature is available in North America region only. Please get in touch with
    [support@nextbillion.ai](mailto:support@nextbillion.ai) to enquire/enable other
    areas.
    """

    departure_time: int
    """Requires option=flexible.

    Use this parameter to set a departure time, expressed as UNIX epoch timestamp in
    seconds, for calculating the isochrone contour. The response will consider the
    typical traffic conditions at the given time and return a contour which can be
    reached under those traffic conditions.

    Please note that if no input is provided for this parameter then the traffic
    conditions at the time of making the request are considered.
    """

    drive_time_limits: str
    """Requires option=flexible.

    An array of durations, in seconds, for which the driver can drive continuously
    before taking a rest. Multiple drive time limits can be separated by a comma
    character ",". After driving for the given duration the driver will take a rest
    for a fixed period, specified in rest_times . Once the rest duration is over,
    the subsequent driving duration starts and the process continues until all drive
    times and rest periods are exhausted or if the driver reaches the destination.
    This feature is useful in complying with Hours of Service regulations and
    calculates actual ETAs with regulated driving periods.

    As an example, a drive_time_limits=\\[[500, 300\\]] means that driver can drive for
    500 seconds before the first rest period and then drive for another 300 seconds
    before taking a rest next time.

    \\-- If the trip duration is smaller than the first input of drive_time_limits,
    then there will be no rest actions scheduled by the service.
    \\-- If the trip duration is larger than the scheduled time, then a "warning" is
    returned in the response - along with details of last leg of the trip - to
    indicate the same.
    """

    emission_class: Literal["euro0", "euro1", "euro2", "euro3", "euro4", "euro5", "euro6", "euro7", "euro8", "euro9"]
    """Requires option=flexible.

    Specify the emission class to which the vehicle (engine) belongs to. The service
    will use this setting to generate routes that are permissible for that engine
    class. Only the emission classifications in the EU regions are supported
    currently. Please reach out to
    [support@nextbillion.ai](mailto:support@nextbillion.ai) to enable for your
    region.
    """

    exclude: Literal[
        "toll", "ferry", "highway", "service_road", "uturn", "sharp_turn", "left_turn", "right_turn", "none"
    ]
    """Requires option=flexible.

    This parameter serves as a mandatory filter, ensuring the service returns only
    those routes that strictly avoid the object(s) indicated. Multiple values should
    be separated by a pipe `|`). If no routes can be found that exclude the
    specified object(s), the service will return an error. For a less strict
    filtering approach, consider using the avoid parameter.

    \\-- This parameter is effective only when route_type=fastest.
    \\-- When using exclude=sharp_turn, default range of permissible turn angles is
    \\[[120,240\\]]. In order to override default range, please use turn_angle_range
    parameter.
    \\-- If none is provided along with other values, an error is returned as a valid
    route is not feasible.
    """

    geometry: Literal["polyline", "polyline6", "geojson"]
    """Sets the output format of the route geometry in the response.

    On providing polyline and polyline6 as input, respective encoded geometry is
    returned. However, when geojson is provided as the input value, polyline encoded
    geometry is returned in the response along with the geojson details of the
    route.
    """

    hazmat_type: Literal["general", "circumstantial", "explosive", "harmful_to_water"]
    """Requires option=flexible.

    Specify the type of hazardous material being carried and the service will avoid
    roads which are not suitable for the type of goods specified. Multiple values
    can be separated using a pipe operator "|".

    Please note that this parameter is effective only when mode=truck.
    """

    mode: Literal["car", "truck"]
    """Set the driving mode the service should use to determine a route.

    In "car" mode, the API will return a route that a car can take. Using "truck"
    mode will return a route a truck can use, taking into account appropriate truck
    routing restrictions.

    When mode=truck, following are the default dimensions that are used:

    \\-- truck_height = 214 centimeters
    \\-- truck_width = 183 centimeters
    \\-- truck_length = 519 centimeters
    \\-- truck_weight = 5000 kg

    When option=flexible, you can use custom truck dimensions with truck_weight and
    truck_size parameters.

    Note: Only the car profile is enabled by default. Please note that customized
    profiles (including truck) might not be available for all regions. Please
    contact your [NextBillion.ai](http://NextBillion.ai) account manager, sales
    representative or reach out at
    [support@nextbillion.ai](mailto:support@nextbillion.ai) in case you need
    additional profiles.
    """

    option: Literal["fast", "flexible"]
    """The option parameter specifies the version of the directions service to use.

    Setting option=flexible activates the Flexible API, which supports advanced
    features like truck routing, time-based routing, route type selection
    (fastest/shortest), and segment-wise speed limits. If not set, the API defaults
    to the Fast version for real-time routing.
    """

    overview: Literal["full", "simplified", "false"]
    """Specify the verbosity of route geometry.

    When set to full, the most detailed geometry available is returned. When set to
    simplified, a simplified version of the full geometry is returned. No overview
    geometry is returned when set to false.
    """

    rest_times: str
    """Requires option=flexible.

    An array of durations, in seconds, for which the driver should rest after
    completing the corresponding continuous driving interval (provided in
    drive_time_limits). Multiple rest times can be separated by a comma character
    ",". Ideally, the number of rest_times provided should be equal to the number of
    drive_time_limits provided for proper scheduling of driver breaks.

    As an example, a rest_times=\\[[500, 300\\]] means that driver can rest for 500
    seconds after the first continuous driving session and rest for 300 seconds
    after the next continuous driving session.

    \\-- If the number of rest_times provided are less than the number of
    drive_time_limits, the service will schedule a rest period of "0" seconds after
    each such drive time period which does not have a corresponding entry in
    rest_times.
    \\-- If the number of rest_times provided is more than the number of drive times
    provided, the additional rest times are never applied.
    """

    road_info: Literal["max_speed", "toll_distance", "toll_cost"]
    """Requires option=flexible.

    Use this parameter to receive additional information about the road segments
    returned in the response. Currently, following inputs are supported:

    \\-- max_speed : segment-wise maximum speed information of roads in the
    response.
    \\-- toll_distance : returns the total distance travelled on the road segments
    having tolls.
    \\-- toll_cost: returns the range of toll charges, in local currency, that can be
    incurred for the suggested route.
    """

    route_type: Literal["fastest", "shortest"]
    """Requires option=flexible.

    Set the route type that needs to be returned.
    """

    steps: bool
    """
    Set this to true to receive additional details about the routes and each of its
    legs (details of geometry, start & end locations) in the response.
    """

    truck_axle_load: float
    """Requires option=flexible.

    Specify the total load per axle (including the weight of trailers and shipped
    goods) of the truck, in tonnes. When used, the service will return routes which
    are legally allowed to carry the load specified per axle.

    Please note this parameter is effective only when mode=truck.
    """

    truck_size: str
    """Requires option=flexible.

    This defines the dimensions of a truck in centimeters (CM). This parameter is
    effective only when the mode=truck. Maximum dimensions are as follows:

    \\-- Height = 1000 cm
    \\-- Width = 5000 cm
    \\-- Length = 5000 cm
    """

    truck_weight: int
    """Requires option=flexible.

    This parameter defines the weight of the truck including trailers and shipped
    goods in kilograms (KG). This parameter is effective only when mode=truck.
    """

    turn_angle_range: int
    """Requires option=flexible.

    Specify the turn angles that can be taken safely by the vehicle. The permissible
    turn angles are calculated as \\[[0 + turn_angle_range , 360 - turn_angle_range\\]].
    Please note that this parameter is effective only when avoid=sharp_turn.

    It is worth highlighting here that providing smaller angles might lead to 4xx
    errors as route engine might not be able find routes satisfying the smaller turn
    angle criteria for all turns in the route.
    """

    waypoints: str
    """Pipe-separated list of coordinate pairs"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["NamespacedApikeyCreateResponse", "Result"]


class Result(BaseModel):
    apikey: Optional[str] = None
    """Returns the unique `key` created for the specified namespace."""

    created_at: Optional[int] = None
    """
    Returns the time, expressed as UNIX epoch timestamp in seconds, when the
    namespace key was created.
    """

    expires_at: Optional[int] = None
    """
    Returns the time, expressed as UNIX epoch timestamp in seconds, when the
    namespace key will expire.
    """

    namespace: Optional[str] = None
    """Returns the name of the `namespace` for which the key is created."""

    sub_id: Optional[str] = None
    """An internal subscription ID."""


class NamespacedApikeyCreateResponse(BaseModel):
    error: Optional[str] = None
    """Returns the error type in case of any error.

    If there is no error, then this field is absent in the response.
    """

    message: Optional[str] = None
    """Returns the error message in case of any error.

    If there is no error, then this field is absent in the response.
    """

    result: Optional[Result] = None
    """An object to return the details about the namespace key created."""

    status: Optional[int] = None
    """Returns the API response code."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["V2RetrieveResultResponse", "Result", "ResultRoute", "ResultRouteStep", "ResultSummary", "ResultUnassigned"]


class ResultRouteStep(BaseModel):
    id: Optional[str] = None
    """Returns the ID of the task.

    The ID returned here are the same values that were provided for the given task
    in the `jobs` or the `shipments` objects of the input POST optimization request.

    **Note:** Since both integer and string value types are supported for job IDs,
    the value type returned for this field will depend on the value type provided in
    the input request.
    """

    arrival: Optional[int] = None
    """Returns the time at which the vehicle arrives at the `step` location.

    If `time_windows` is provided for the task it will be returned as an UNIX
    timestamp expressed in seconds. When `time_windows` is not provided, it is
    returned as the total duration, in seconds, elapsed since the start of the
    route.

    Please note it includes all the other durations as well (setup, service,
    waiting).
    """

    depot: Optional[str] = None
    """
    For step type "start" or "end", this field returns the ID of the depot relevant
    to that step. For "start" steps, the field will contain the ID of the depot from
    which the vehicle commenced its journey. Conversely, for "end" steps, the field
    will hold the ID of the depot where the vehicle concluded its trip.

    Please note that `start_depot_ids` or `end_depot_ids` input for the vehicle must
    be configured to get this field in the response for respective step types in a
    route.
    """

    description: Optional[str] = None
    """Returns the description of this step.

    The description returned here are the same values that were provided for the
    given task in the `jobs` or the `shipments` objects of the input POST
    optimization request.
    """

    distance: Optional[int] = None
    """
    Returns the distance covered, in meters, from the start of the route and up
    until the current step.

    Please note that the value of this parameter accumulates with each step. In case
    , the `travel_cost: air_distance`, then the distance here represents straight
    line distance.
    """

    duration: Optional[int] = None
    """
    Returns the total drive time, in seconds, from the start of the route up until
    the start of the `step`. Please note that this value does not include any other
    category of durations (service, wait, setup) and the value of this parameter
    accumulates with each step.
    """

    late_by: Optional[str] = None
    """
    Returns the amount of time, in seconds, by which the vehicle is late when
    arriving at this step. Please note this field is present only when there is a
    non-zero value for vehicle lateness, otherwise it is not present in the
    response.
    """

    load: Optional[List[int]] = None
    """Returns the load on the vehicle after completing this step.

    In case of multiple dimensions, loads of each type are returned by a matching
    number of elements in the array.
    """

    location: Optional[List[float]] = None
    """
    Returns the location coordinates of the step in the \\[[latitude, longitude\\]]
    format.

    The index of this location is also returned by the `location_index` parameter.
    """

    location_index: Optional[int] = None
    """
    Returns the index (in the `location` array) of the location coordinates where
    the step is performed. The index will always be in the range of \\[[0, length of
    `location` array).

    Actual coordinates are also returned by the `location` parameter.
    """

    metadata: Optional[object] = None
    """
    Returns the custom information that was provided when the given task (job /
    pickup / delivery) was configured. This field would not be present for the tasks
    that were not provided with any metadata. It will also be not present for
    “start” and “end” steps.
    """

    projected_location: Optional[List[float]] = None
    """
    In case this step is part of a task group, this field returns the location
    coordinates of the point, in \\[[latitude, longitude\\]] format, which was used as a
    common stop for all grouped tasks.
    """

    run: Optional[int] = None
    """
    When a vehicle is configured to make multiple runs to the depot (via
    `max_depot_runs`), this field returns the iteration to which the step belongs
    to. Each run will begin with a "start" step from the depot's location and
    conclude with an "end" step at either the last task's or the configured end
    location.
    """

    service: Optional[int] = None
    """
    Returns the service time, in seconds, for the task when the step type is not
    `start` or `end`.

    When the step type is `start` or `end` , the field also returns the service
    time, in seconds, spent at the depot when if the vehicle is starting or
    completing the trip at one of the depots.
    """

    setup: Optional[int] = None
    """Returns the setup time, in seconds, for the task."""

    snapped_location: Optional[List[float]] = None
    """
    Returns the coordinates after snapping the location of this step to a nearby
    road. Please note that this field will not be available in the response when
    custom duration or distance matrix were used for cost calculations.
    """

    type: Optional[str] = None
    """Returns the type of the step.

    Its value will always be one of the following: `start`, `job`, `pickup`,
    `delivery`, `end`. In case the type is `start` or `end`, `steps` object will not
    have the `id` field.
    """

    waiting_time: Optional[int] = None
    """Returns the wait time of the vehicle at this step, in seconds."""


class ResultRoute(BaseModel):
    adopted_capacity: Optional[List[int]] = None
    """Returns the capacity configuration of the vehicle that was used for this route.

    This field would return either the vehicle's `capacity` or one of the
    `alternative_capacities` provided in the input request.
    """

    cost: Optional[int] = None
    """Returns the cost of the route.

    The unit of cost type depends on the value of `travel_cost` attribute in the
    optimization request.
    """

    delivery: Optional[List[int]] = None
    """
    Returns the total quantities, for each dimension (or unit), of deliveries
    performed in the route. Please note that when both `shipments` and `jobs` are
    provided, this field corresponds to the sum of quantities delivered as part of
    the assigned `shipments` and `jobs` on the route.
    """

    description: Optional[str] = None
    """Return the description of the assigned vehicle.

    It would be the same as that provided in the `description` field of `vehicles`
    part of the input POST optimization request.
    """

    distance: Optional[float] = None
    """Returns the total distance of the route, in meters."""

    duration: Optional[int] = None
    """Returns the total drive duration of the route, in seconds."""

    geometry: Optional[str] = None
    """Returns the geometry of this route encoded in polyline format."""

    metadata: Optional[object] = None
    """Returns the custom information that was provided when the vehicle was
    configured.

    This field would not be present for the vehicles that were not provided with any
    metadata.
    """

    pickup: Optional[List[int]] = None
    """
    Returns the total quantities, for each dimension (or unit), of pickups performed
    in the route. Please note that when both `shipments` and `jobs` are provided,
    this field corresponds to the sum of quantities picked-up as part of the
    assigned `shipments` and `jobs` on the route.
    """

    priority: Optional[int] = None
    """Returns the sum of priorities of all tasks on the route."""

    profile: Optional[str] = None
    """Returns the profile of the vehicle."""

    revenue: Optional[int] = None
    """Returns the revenue earned by fulfilling the task on this route.

    Please note this field is present only when the revenue inputs are provided in
    the input, otherwise it is not present in the response.
    """

    service: Optional[int] = None
    """
    Returns the total service time spent on the tasks or depots on the route, in
    seconds.
    """

    setup: Optional[int] = None
    """Returns the total setup time, in seconds, for the tasks assigned on the route."""

    steps: Optional[List[ResultRouteStep]] = None
    """This attribute contains the details of all the steps involved in the route.

    It is an array of objects with each object representing one step.
    """

    vehicle: Optional[str] = None
    """Returns the ID of the vehicle that was assigned to the route.

    The value type will be same as the value type provided in the input request.
    """

    vehicle_overtime: Optional[int] = None
    """Returns the total vehicle overtime for the route, in seconds.

    Please note this field is present only when there is a non-zero value for
    vehicle overtime, otherwise it is not present in the response.
    """

    waiting_time: Optional[int] = None
    """Returns the total waiting time of the vehicle on the route, in seconds."""


class ResultSummary(BaseModel):
    cost: Optional[int] = None
    """Returns the total cost of all the routes returned in the solution.

    The unit of cost type depends on the value of `travel_cost` attribute in the
    optimization request.
    """

    delivery: Optional[List[int]] = None
    """Returns the sum of all quantities that were delivered in the optimized solution.

    If quantities of different dimensions were delivered, then a matching number of
    elements is returned in the `delivery` array.

    Please note that when both `shipments` and `jobs` are provided, this field
    corresponds to the sum of quantities delivered as part of all the assigned
    `shipments` and `jobs` .
    """

    distance: Optional[float] = None
    """Returns the total distance of all routes, in meters.

    It is equal to the sum of distances of individual routes.
    """

    duration: Optional[int] = None
    """Returns the total drive time, in seconds, needed to cover all routes.

    Please note that it does not include the service, setup or the waiting durations
    elapsed on these routes.
    """

    num_late_visits: Optional[int] = None
    """
    Returns the total number of tasks across all routes that failed to start within
    their scheduled time windows.
    """

    pickup: Optional[List[int]] = None
    """Returns the sum of all quantities that were picked-up in the optimized solution.

    If quantities of different dimensions were picked-up, then a matching number of
    elements is returned in the `pickup` array.

    Please note that when both `shipments` and `jobs` are provided, this field
    corresponds to the sum of quantities picked-up as part of all the assigned
    `shipments` and `jobs` .
    """

    priority: Optional[int] = None
    """Returns the sum of priorities of all tasks that were assigned."""

    revenue: Optional[int] = None
    """Returns the revenue earned by completing all the assigned tasks.

    Overall profit earned by following the suggested route plan can be inferred by
    subtracting the `cost` of the solution from the reported `revenue`.
    """

    routes: Optional[int] = None
    """Returns the total number of routes in the solution."""

    service: Optional[int] = None
    """Returns the total service time, in seconds, for all the routes in the solution.

    It is equal to the sum of service time of individual tasks that were assigned
    and the time spent loading/unloading items at designated depots by all vehicles.
    """

    setup: Optional[int] = None
    """Returns the total setup time, in seconds, of all assigned tasks.

    It is equal to the sum of setup time of individual tasks that were assigned.
    """

    total_visit_lateness: Optional[int] = None
    """
    Returns the total duration, in seconds, that tasks across all routes were
    delayed from starting after their scheduled time windows had passed.
    """

    unassigned: Optional[int] = None
    """Returns the number of unfulfilled tasks in the solution."""

    waiting_time: Optional[int] = None
    """
    Returns the sum of durations spent waiting, in seconds, by vehicles on all
    routes.
    """


class ResultUnassigned(BaseModel):
    id: Optional[str] = None
    """Returns the ID of the unassigned task.

    The ID returned is the same as that provided for the given task in the `jobs` or
    the `shipments` part in the input POST optimization request.

    **Note:** Since both integer and string value types are supported for task IDs,
    the value type returned for this field will depend on the value type provided in
    the input request for the unassigned task.
    """

    location: Optional[List[float]] = None
    """
    Returns the location of the unassigned tasks in the \\[[latitude, longitude\\]]
    format.
    """

    outsourcing_cost: Optional[int] = None
    """Returns the cost of outsourcing the task.

    This is the same value as provided in the input. The field is present only if a
    `outsourcing_cost` was provided for the unassigned task.
    """

    reason: Optional[str] = None
    """Returns the most likely reason due to which the task remained unassigned.

    The optimization service can capture the following causes of tasks remaining
    unassigned, among others:

    - unmatched `skills` of the tasks
    - insufficient `capacity` of vehicle to accommodate the tasks
    - `time_window` requirements of the tasks or the vehicles
    - violation of vehicle’s `max_activity_waiting_time` constraint
    - violation of vehicle’s `max_tasks` or `max_stops` constraints
    - violation of vehicle’s `max_distance` or `max_travel_time` constraints
    - task unassigned due to zone constraints
    - task unassigned due to depot constraints
    - task unassigned due to load type incompatibility constraints
    - task unassigned due to max time in vehicle constraint
    - task unassigned as it is unprofitable
    - task unassigned due to low outsourcing cost
    - task unassigned due to infeasible conditions specified in `relations`
      attribute
    """

    type: Optional[str] = None
    """Returns the type of the task that was unassigned.

    Will always belong to one of `job`, `pickup`, or `delivery`.
    """


class Result(BaseModel):
    code: Optional[int] = None
    """A custom code representing the status of the result.

    A code other than 0, represents an internal error. In case of codes other than
    0, please verify the parameter values, constraints and locations. If the issue
    does not resolve, please reach out to NextBillion at
    [support@nextbillion.ai](mailto:support@nextbillion.ai).
    """

    error: Optional[str] = None
    """Returns the error message for unfulfilled optimization jobs.

    This field will not be present in the response in case there are no errors.
    """

    routes: Optional[List[ResultRoute]] = None
    """
    An array of objects containing the details of each route in the optimized
    solution. Each object represents one route.
    """

    routing_profiles: Optional[object] = None
    """Returns all the routing profiles used in the solution.

    If no routing profiles were provided in the input or if the vehicles tagged to
    profiles were not used in the solution, the "default" routing properties are
    returned. Default routing properties are indicated by `options.routing` in the
    input.
    """

    summary: Optional[ResultSummary] = None
    """An object to describe the summarized result of the optimization request.

    This object can be useful to quickly get an overview of the important result
    parameters.
    """

    unassigned: Optional[List[ResultUnassigned]] = None
    """
    An array of objects containing the details of unassigned tasks in the optimized
    solution. Each object represents one task.
    """


class V2RetrieveResultResponse(BaseModel):
    description: Optional[str] = None
    """
    Returns the `description` of the optimization job as given in the input POST
    optimization request. This field will not be present in the response if no
    `description` was provided in the input request.
    """

    message: Optional[str] = None
    """
    Returns the message in case of errors or failures, otherwise a blank string is
    returned.
    """

    result: Optional[Result] = None
    """An object containing the details of the optimized routes."""

    status: Optional[Literal["`Ok`", "`Error`"]] = None
    """
    It indicates the overall status or result of the API request denoting whether
    the operation was successful or did it encounter any errors.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["ShipmentParam", "Delivery", "Pickup", "Volume"]


class Delivery(TypedDict, total=False):
    id: Required[str]
    """Indicate the ID of this shipment delivery step.

    An error will be reported if there are duplicate IDs for multiple shipment
    deliveries. The IDs are case sensitive. Please note `id` is mandatory when using
    the `shipments` attribute.
    """

    location_index: Required[int]
    """Indicate the index of location for this shipment delivery.

    The index references the locations present in the `location` array. The valid
    range of value is \\[[0, length of `location` array).

    Please note `location_index` is mandatory when using the `shipment` attribute.
    """

    description: str
    """Specify a custom description for the shipment delivery step."""

    max_visit_lateness: int
    """
    It determines the allowable delay, in seconds, to begin a shipment delivery
    after its designated time window has concluded. Please note that this parameter
    would override the `constraint.max_visit_lateness` (global) if both are
    specified.
    """

    metadata: object
    """
    Specify any custom data that should be attached along with delivery fulfilment
    details in the `step` attribute of the optimized solution. Users can leverage
    this property to provide additional details/context when sharing information
    about the delivery step with integrated systems (TMS, Fleet Management, Driver
    dispatch etc).

    Please note that the `metadata` content must always be specified in a `key` :
    `value` pair format, where the “key” is always a string.
    """

    sequence_order: int
    """
    Use this parameter to prioritize completing the shipment delivery relative to
    certain other tasks. A task configured with a `sequence_order` of 2 will be done
    after the task with `sequence_order` of 1, but before the task with
    `sequence_order` of 3. Valid range of values for this input is \\[[0,100\\]].

    Please note that:

    - The shipment delivery's sequence order must be greater than or equal to its
      corresponding pickup's sequence order.
    - Only the tasks within the same route are compared and ordered as per their
      `sequence_order`.
    - Tasks without a `sequence_order` are not involved in the comparison.
    - Following is the precedence of `sequence_order` when used along side some of
      the other constraints:

      - `relations` are prioritized over `sequence_order` comparisons.
      - `sequence_order` will override `order_grouping` configurations.
    """

    service: int
    """Provide the time duration, in seconds, needed to complete the shipment delivery.

    Default value is 0.
    """

    setup: int
    """Specify the set-up duration, in seconds, for the delivery.

    `setup` is the one-time effort needed apart from working on the original task-
    for example, effort to record some information for compliance, or effort to
    set-up the equipment, or perform any other action for completing all steps
    required to fulfil the job.

    Please note that `setup` time is applied only once for a given task location.
    `setup` time, unlike `service` time, is not repeated in case there are multiple
    tasks at the same location.
    """

    time_windows: Iterable[Iterable[int]]
    """Describe time periods within which the shipment delivery should start.

    The time periods should be expressed as a UNIX timestamp in seconds.

    Please note that the time periods should not overlap with each other and should
    always follow the format of \\[[start_timestamp, end_timestamp\\]].
    """


class Pickup(TypedDict, total=False):
    id: Required[str]
    """Indicate the ID of this shipment pickup step.

    An error will be reported if there are duplicate IDs for multiple shipment
    pick-ups. The IDs are case-sensitive. Please note `id` is mandatory when using
    the `shipments` attribute.
    """

    location_index: Required[int]
    """Indicate the index of the location for this shipment pickup.

    The index references the locations present in the `location` array. The valid
    range of value is \\[[0, length of `location` array).
    """

    description: str
    """Specify a custom description for the shipment pickup step."""

    max_visit_lateness: int
    """
    It determines the allowable delay, in seconds, to begin a shipment pickup after
    its designated time window has concluded. Please note that this parameter would
    override the `constraint.max_visit_lateness` (global) if both are specified.
    """

    metadata: object
    """
    Specify any custom data that should be attached along with pickup fulfilment
    details in the `steps` attribute of the optimized solution. Users can leverage
    this property to provide additional details/context when sharing information
    about the pickup step with integrated systems (TMS, Fleet Management, Driver
    dispatch etc).

    Please note that the `metadata` content must always be specified in a`key` :
    `value` pair format, where the “key” is always a string.
    """

    sequence_order: int
    """
    Use this parameter to prioritize completing the shipment pickup relative to
    certain other tasks. A task configured with a `sequence_order` of 2 will be done
    after the task with `sequence_order` of 1, but before the task with
    `sequence_order` of 3. Valid range of values for this input is \\[[0,100\\]].

    Please note that:

    - The shipment pickups's sequence order must be lesser than or equal to its
      corresponding delivery's sequence order.
    - Only the tasks within the same route are compared and ordered as per their
      `sequence_order`.
    - Tasks without a `sequence_order` are not involved in the comparison.
    - Following is the precedence of `sequence_order` when used along side some of
      the other constraints:

      - `relations` are prioritized over `sequence_order` comparisons.
      - `sequence_order` will override `order_grouping` configurations.
    """

    service: int
    """Provide the time duration, in seconds, needed to complete the shipment pickup.

    Default value is 0.
    """

    setup: int
    """Specify the set-up duration, in seconds, for the pickup.

    `setup` is the one-time effort needed apart from working on the original task-
    for example, effort to record some information for compliance, or effort to
    set-up the equipment, or perform any other action for completing all steps
    required to fulfil the job.

    Please note that `setup` time is applied only once for a given task location.
    `setup` time, unlike `service` time, is not repeated in case there are multiple
    tasks at the same location.
    """

    time_windows: Iterable[Iterable[int]]
    """Describe time periods within which the shipment pickup should be start.

    The time periods should be expressed as a UNIX timestamp in seconds.

    Please note that the time periods should not overlap with each other and should
    always follow the format of \\[[start_timestamp, end_timestamp\\]]
    """


class Volume(TypedDict, total=False):
    alignment: Literal["`strict`", "`parallel`", "`fixed_bottom`", '`" "`']
    """Refers to the orientation of the cargo in the loading compartment.

    It supports the following values:

    - **" "** : A blank space to indicate that the dimension check for the cargo
      should be skipped. This is also the default value.
    - **strict :** In this orientation, the cargo must fit within the vehicle’s
      dimensions exactly as is—no rotation is allowed. All dimensions of the cargo
      must be less than or equal to the corresponding dimensions of the vehicle. If
      any dimension exceeds that of the vehicle, the cargo cannot be loaded.
    - **parallel :** With parallel orientation, the cargo can be rotated around any
      one of the axes to help it fit into the loading compartment. For example, if
      the cargo is wider than the vehicle, it can still be loaded by rotating it
      around the vertical axis (so the width aligns with the vehicle’s depth) or the
      horizontal axis (so the width aligns with the vehicle’s height). Using this
      orientation, even a cargo whose one dimension is larger than the corresponding
      dimension of the vehicle but smaller than other dimensions, can also be
      loaded.
    - **fixed_bottom :** In this orientation, the cargo can only be rotated around
      the vertical axis, meaning its base stays fixed and it cannot be tilted or
      turned over. The height of the cargo remains aligned with the vehicle’s height
      and cannot be adjusted. As a result, if the cargo’s height exceeds the
      vehicle’s available height, it cannot be loaded into the compartment.
    """

    depth: float
    """Cargo length, in meters."""

    height: float
    """Cargo height, in meters."""

    width: float
    """Cargo width, in meters."""


class ShipmentParam(TypedDict, total=False):
    delivery: Required[Delivery]
    """Specify the details of the delivery step of the shipment."""

    pickup: Required[Pickup]
    """Specify the details of the pickup step of the shipment."""

    amount: Iterable[int]
    """This parameter defines the quantity that needs to be shipped.

    This attribute supports multidimensional quantities, to support shipment of
    quantities of different units/dimensions. It is recommended to keep the
    dimensions of `amount` in `shipments` and that of `capacity` in `vehicles`
    consistent.

    Please note that the `amount` will be added to the assigned vehicle’s initial
    load.

    Read more about the behavior of this attribute in the
    [Multi-Dimensional Parameters](#multi-dimensional-parameters) section.
    """

    follow_lifo_order: bool
    """Specify whether the shipment route should follow LIFO (last in, first out).

    Use this parameter when the loading or unloading sequence of cargo is important.
    The default value is \\``false\\``.
    """

    incompatible_load_types: List[str]
    """
    Use this parameter to specify the type of loads which are incompatible with the
    shipment’s load type. Once this property is configured, the shipment can only be
    serviced by a vehicle which has not serviced any other task with an incompatible
    `load_types` . Add multiple load types to indicate all the types which are
    incompatible for this shipment. The incompatible load type considerations are
    ignored for the first task of the route.

    For example, an input value of \\[[“groceries”, “food”\\]] means that shipment’s
    load is incompatible with both groceries and food type of loads. Consequently,
    the optimizer will not assign this shipment to a vehicle which has served any
    task with `load_types` as either groceries or food.

    Please note that if the shipment is part of any `relations` then, configured
    `incompatible_load_types` might be ignored.
    """

    joint_order: int
    """Specify a joint order group ID that this shipment is associated with.

    Tasks having the same joint order group ID are treated as a single unit: either
    all tasks in the group are assigned, or none are. Users can add tasks of both
    `jobs` and `shipments` types to a single joint order group by using the same
    unique ID. Please note that:

    - Each shipment in a single joint order group will be fulfilled by a unique
      vehicle.
    - Shipments belonging to a joint order group can be served in any sequence while
      maintaining the pickup -> delivery sequence for an individual shipment.
    - Joint order settings will not be effective if \\``solution\\`` or \\``relations\\``
      attributes are also provided in the input request.
    """

    load_types: List[str]
    """Use this parameter to specify the type of loads for the given shipment.

    Once this property is configured, the shipment can not be served by a vehicle
    which has serviced any task whose load is incompatible with any of
    the`load_types` provided in this input. The load type considerations are ignored
    for the first task of the route.

    For example, an input value of \\[[“groceries”, “food”\\]] means that shipment’s
    load characteristics belong to either one or both types. Consequently, the
    optimizer will assign this shipment to a vehicle which has served other tasks
    whose `incompatible_load_types` do not contain either groceries or food.

    Please note that if the shipment is part of any `relations` then, configured
    `load_types` might be ignored.
    """

    max_time_in_vehicle: int
    """
    Use this parameter to limit the drive time for which a shipment stays in the
    vehicle. The time-in-vehicle calculations start once the pickup leg of shipment
    is completed after serving any setup and service time that may have been
    configured for it. For the delivery leg, time-in-vehicle calculations wouldn’t
    consider any setup and service time that needs to be served for completing the
    delivery. The service or setup times of other tasks performed in between will
    also be not accumulated against the time-in-vehicle limit.

    Please note that this property would be overridden if any `relations`
    configuration is used except for “precedence” type. If “precedence” type
    relations is used then `max_time_in_vehicle` will override it.
    """

    outsourcing_cost: int
    """
    Specify the cost of keeping this shipment unassigned, namely, the cost of
    outsourcing the shipment. When provided, the optimizer weighs the cost of
    assigning the shipment against the cost of keeping it unassigned and chooses a
    solution with lower cost. In the solution, the `outsourcing_cost` of unassigned
    shipments is added to the total cost of the solution.

    If the `outsourcing_cost` is not provided, which is to say that the shipment can
    not be outsourced, then the optimizer tries to fulfill the shipment irrespective
    of the cost incurred, subject to other constraints.

    Please note that `revenue` and `outsourcing_cost` can not be specified
    simultaneously for a shipment. Also, the `outsourcing_cost` would override the
    priority settings of the shipment.
    """

    priority: int
    """Describe the priority of this shipment.

    The valid values are in the range of \\[[0, 100\\]]. Default value is 0.

    Please note that setting a priority will only decide whether this shipment will
    be assigned or not, but has nothing to do with the sequence of fulfilling
    shipments.
    """

    revenue: int
    """Specify the revenue earned by completing this shipment.

    The optimizer uses the `revenue` input to identify the potential profit earned
    by fulfilling this shipment after taking into account the costs incurred to do
    so. The`options.objective` and `vehicles.costs` input are taken into account to
    identify the costs of fulfilling the shipment.

    In general, the optimizer will prefer fulfilling the tasks with higher profits
    over the tasks with lower profits, should it need to reject some tasks in order
    to honor other constraints. In case the profit from fulfilling a shipment is
    negative, it will remain unassigned whatsoever.
    """

    skills: Iterable[int]
    """Define the skills needed to complete the shipment.

    This attribute supports multidimensional skills allowing users to add multiple
    skills for a shipment.

    Read more about the behavior of this attribute in the
    [Multi-Dimensional Parameters](#multi-dimensional-parameters) section.
    """

    volume: Volume
    """
    Specify the dimensions and alignment configurations for the cargo associated
    with the shipment. These inputs will be used to arrange the items into the
    loading compartment of the vehicle to utilize the three-dimensional space. If a
    shipment consists of several different items, each with its own dimensions,
    please specify the final characteristics for the task: total height, total
    depth, total width.

    Please note that vehicles which contain the `volume` input, will only be
    considered for arranging such items.
    """

    zones: Iterable[int]
    """
    An array of integers specifying the IDs of the zone(s) that this shipment
    belongs to. The shipment can be fulfilled by all vehicles which are allowed to
    complete tasks in the zone(s) assigned to this shipment. If the pickup and
    delivery steps belong to different zones, then a vehicle should be allowed to
    fulfil tasks in both zones to take up such shipments. Please note following
    points about shipment zones:

    - If zone IDs are provided for any one of the shipments, then all other
      shipments should also be specified with zone IDs. Zone IDs provided here will
      override any zone geometries provided in the `zones` attribute and these IDs
      will be used for allocating appropriate vehicles.
    - Shipment steps can be auto-allocated to zones if this parameter is not
      specified while the zone geometries (either `zones.geometry` or
      `zones.geofence_id`) are provided.
    - Shipments not falling in any zones can be fulfilled by only those vehicles
      which are allowed to take up tasks outside zones as well. Refer to `vehicles`
      attribute for more information.
    """
